unit FreeOTFEfrmGridReport;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Grids,
  OTFEFreeOTFE_U, Menus, ActnList, ExtCtrls, SDUStringGrid, Buttons;

type
  TTextFormat = (clfText, clfTSV, clfCSV);

  TfrmGridReport = class(TForm)
    pbSave: TButton;
    pbClose: TButton;
    ActionList1: TActionList;
    mnuPopup: TPopupMenu;
    actSelectAll: TAction;
    actCopy: TAction;
    Selectall1: TMenuItem;
    Copy1: TMenuItem;
    lblTitle: TLabel;
    SaveDialog: TSaveDialog;
    stgReport: TSDUStringGrid;
    pnlBetweenButtons: TPanel;
    pnlBetweenButtonsCenter: TPanel;
    lblTotalDrivers: TLabel;
    lblTotalAlgorithms: TLabel;
    ckShowAdvanced: TCheckBox;
    pbbDrivers: TBitBtn;
    procedure pbCloseClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure actSelectAllExecute(Sender: TObject);
    procedure ckShowAdvancedClick(Sender: TObject);
    procedure actCopyExecute(Sender: TObject);
    procedure pbSaveClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure stgReportMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure pbbDriversClick(Sender: TObject);
  private
    procedure ResizeGridCols();

    procedure ShowHideColumns(displayAdvanced: boolean);

    function  FormatRow(format: TTextFormat; rowIdx: integer): string;
    function  FormatRow_TSV(rowIdx: integer): string;
    function  FormatRow_CSV(rowIdx: integer): string;
    function  FormatRow_xSV(rowIdx: integer; seperatorChar: char): string;
    function  FormatRow_Text(rowIdx: integer): string;
  protected
    function CountDrivers(): integer; virtual; abstract;
    function CountImplementations(): integer; virtual; abstract;

    procedure SetupGrid(); virtual;
    procedure PopulateGrid(); virtual; abstract;
    function  IsColumnAdvanced(colIdx: integer): boolean; virtual;
  public
    OTFEFreeOTFE: TOTFEFreeOTFE;
  end;

const
  GRID_HEADER_ROW = 0;

implementation

{$R *.dfm}

uses
  Clipbrd, // Required for clipboard functions
  Math,
  SDUGeneral,
  SDUDialogs,
  FreeOTFEfrmMain;

const
  // Save dialog related
  FILE_FILTER_FLT_REPORTS  = 'Text report (*.txt)|*.txt|Comma separated values (*.csv)|*.csv|Tab separated values (*.tsv)|*.tsv';
  FILE_FILTER_DFLT_REPORTS = 'txt';
  // Important! This ordering maps the TTextFormat to the filter index
  // specified in FILE_FILTER_FLT_REPORTS
  FILE_FILTER_TO_FORMAT: array [TTextFormat] of integer = (1, 3, 2);

procedure TfrmGridReport.SetupGrid();
begin
  // Crude, but better than the Delphi default...
  stgReport.DefaultRowHeight := trunc(stgReport.font.size * 2.5);

  stgReport.RowCount  := 2; // To be setup correctly in PopulateGrid(...)
  stgReport.FixedRows := 1;
  stgReport.FixedCols := 0;

  stgReport.Options := stgReport.Options +
                       [
                        goRowSelect,
                        goColSizing,
                        goRowSizing
                       ];

end;

procedure TfrmGridReport.ResizeGridCols();
var
  i: integer;
begin
  for i:=0 to (stgReport.ColCount - 1) do
    begin
    stgReport.AutoSizeGridColumn(i);
    end;
end;

procedure TfrmGridReport.actCopyExecute(Sender: TObject);
var
  outString: string;
  selection: TGridRect;
  i: integer;
begin
  outString := '';
  selection := stgReport.Selection;

  outString := outString + FormatRow(clfTSV, GRID_HEADER_ROW) + SDUCRLF;
  for i:=selection.Top to selection.Bottom do
    begin
    outString := outString + FormatRow(clfTSV, i) + SDUCRLF;
    end;

  Clipboard.AsText := outString;

end;

procedure TfrmGridReport.actSelectAllExecute(Sender: TObject);
var
  selection: TGridRect;
begin
  selection.Top := GRID_HEADER_ROW + 1; // First one after the headers
  selection.Bottom := stgReport.Rowcount - 1; // -1 to skip the headers
  selection.Left := 0;
  selection.Right := stgReport.ColCount - 1; // -1 as index from zero

  stgReport.Selection := selection;

end;

procedure TfrmGridReport.ckShowAdvancedClick(Sender: TObject);
begin
  ShowHideColumns(ckShowAdvanced.checked);
end;

procedure TfrmGridReport.ShowHideColumns(displayAdvanced: boolean);
var
  i: integer;
begin
  for i:=0 to (stgReport.ColCount - 1) do
    begin
    if IsColumnAdvanced(i) then
      begin
      stgReport.ColVisible[i] := displayAdvanced;
      end;
    end;

end;

procedure TfrmGridReport.stgReportMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var
  colIdx: integer;
  rowIdx: integer;
begin
  stgReport.MouseToCell(X, Y, colIdx, rowIdx);
  if (
      (rowIdx >= 0) and
      (rowIdx < stgReport.FixedRows)
     ) then
    begin
    stgReport.SortGrid(colIdx);
    end;

end;

procedure TfrmGridReport.FormResize(Sender: TObject);
begin
  SDUCenterControl(pnlBetweenButtonsCenter, ccHorizontal);
end;

procedure TfrmGridReport.FormShow(Sender: TObject);
var
  totalAlgs: integer;
begin
  totalAlgs := CountImplementations();

  lblTotalDrivers.caption    := 'Total drivers: '+inttostr(CountDrivers());
  lblTotalAlgorithms.caption := 'Total algorithms: '+inttostr(totalAlgs);

  SetupGrid();
  if (totalAlgs > 0) then
    begin
    PopulateGrid();
    end;
  ResizeGridCols();

  ckShowAdvanced.checked := FALSE;
  ShowHideColumns(ckShowAdvanced.checked);

end;

procedure TfrmGridReport.pbbDriversClick(Sender: TObject);
begin
  if (Owner is TfrmFreeOTFEMain) then
    begin
    TfrmFreeOTFEMain(Owner).DisplayDriverControlDlg();
    end;
end;

procedure TfrmGridReport.pbCloseClick(Sender: TObject);
begin
  Close();
end;

procedure TfrmGridReport.pbSaveClick(Sender: TObject);
var
  testFormatCypher: TTextFormat;
  useFormat: TTextFormat;
  sContent: string;
  stlContent: TStringList;
  i: integer;
  startRow: integer;
begin
  SaveDialog.Filter     := FILE_FILTER_FLT_REPORTS;
  SaveDialog.DefaultExt := FILE_FILTER_DFLT_REPORTS;

  if SaveDialog.Execute() then
    begin
    // Identify file format to save as...
    useFormat := clfText;
    for testFormatCypher:=low(FILE_FILTER_TO_FORMAT) to high(FILE_FILTER_TO_FORMAT) do
      begin
      if (SaveDialog.FilterIndex = FILE_FILTER_TO_FORMAT[testFormatCypher]) then
        begin
        useFormat := testFormatCypher;
        break;
        end;
      end;

    // Generate file contents...
    startRow := GRID_HEADER_ROW;
    if (useFormat = clfText) then
      begin
      inc(startRow);
      end;

    if (CountImplementations() <= 0) then
      begin
      sContent := 'No implementations found';
      end
    else
      begin
      for i:=startRow to (stgReport.RowCount - 1) do
        begin
        sContent := sContent + FormatRow(useFormat, i);
        end;
      end;

    stlContent:= TStringList.Create();
    try
      // Add header if text report
      if (useFormat = clfText) then
        begin
        OTFEFreeOTFE.AddStdDumpHeader(stlContent, self.caption);
        stlContent.Add('Summary');
        stlContent.Add('-------');
        stlContent.Add('Total drivers   : '+inttostr(CountDrivers()));
        stlContent.Add('Total algorithms: '+inttostr(CountImplementations()));
        stlContent.Add('');
        stlContent.Add('');
        stlContent.Add('Details');
        stlContent.Add('-------');
        stlContent.Add('');
        end;

      // ...and dump to file
      stlContent.Text := stlContent.Text + sContent;
      stlContent.SaveToFile(SaveDialog.FileName);
    finally
      stlContent.Free();
    end;
    end;

end;

function TfrmGridReport.FormatRow_TSV(rowIdx: integer): string;
begin
  Result := FormatRow_xSV(rowIdx, #9);
end;

function TfrmGridReport.FormatRow_CSV(rowIdx: integer): string;
begin
  Result := FormatRow_xSV(rowIdx, ',');
end;

function TfrmGridReport.FormatRow_xSV(rowIdx: integer; seperatorChar: char): string;
var
  retval: string;
  i: integer;
begin
  retval := '';

  for i:=0 to (stgReport.ColCount - 1) do
    begin
    if (retval <> '') then
      begin
      retval := retval + seperatorChar;
      end;

    retval := retval + stgReport.Cells[i, rowIdx];
    end;

  Result := retval;
end;

procedure TfrmGridReport.FormCreate(Sender: TObject);
begin
  pnlBetweenButtons.BevelInner := bvNone;
  pnlBetweenButtons.BevelOuter := bvNone;
  pnlBetweenButtons.Caption := '';
  pnlBetweenButtonsCenter.BevelInner := bvNone;
  pnlBetweenButtonsCenter.BevelOuter := bvNone;
  pnlBetweenButtonsCenter.Caption := '';

  if SDUSetUACShieldIcon(pbbDrivers) then
    begin
    pbbDrivers.width := pbbDrivers.width + pbbDrivers.Glyph.width;
    end;

end;

function TfrmGridReport.FormatRow_Text(rowIdx: integer): string;
var
  retval: string;
  i: integer;
  maxTitleX: integer;
begin
  retval := '';

  // Identify longest column title
  maxTitleX := 0;
  for i:=0 to (stgReport.ColCount - 1) do
    begin
    maxTitleX := max(maxTitleX, length(stgReport.Cells[i, GRID_HEADER_ROW]));
    end;

  for i:=0 to (stgReport.ColCount - 1) do
    begin
    retval := retval +
              stgReport.Cells[i, GRID_HEADER_ROW] +
              StringOfChar(' ', (maxTitleX - length(stgReport.Cells[i, GRID_HEADER_ROW]))) +
              ': ' +
              stgReport.Cells[i, rowIdx] +
              SDUCRLF;
    end;

  retval := retval + SDUCRLF;

  Result := retval;
end;

function TfrmGridReport.FormatRow(format: TTextFormat; rowIdx: integer): string;
var
  retval: string;
begin
  retval := '';

  case format of
    clfTSV:
      begin
      retval := FormatRow_TSV(rowIdx);
      end;

    clfCSV:
      begin
      retval := FormatRow_CSV(rowIdx);
      end;

    clfText:
      begin
      retval := FormatRow_Text(rowIdx);
      end;

    else
      begin
      SDUMessageDlg(
                    'Unknown output format?!'+SDUCRLF+SDUCRLF+'Please report seeing this error!',
                    mtError,
                    [mbOK],
                    0
                   );
      end;

    end;

  Result := retval;
end;

function TfrmGridReport.IsColumnAdvanced(colIdx: integer): boolean;
begin
  Result := FALSE;
end;


END.

