unit FreeOTFEfrmGridReport_Cypher;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, FreeOTFEfrmGridReport, Menus, ActnList, StdCtrls, Grids, ExtCtrls,
  SDUStringGrid, Buttons;

type
  TGridColumn_Cypher = (
                        gccDriverTitle,
                        gccDriverVersion,
                        gccDriverGUID,
                        gccDriverDeviceName,
                        gccDriverUserModeName,
                        gccDriverKernelModeName,

                        gccCypherTitle,
                        gccCypherKeysizeUnderlying,
                        gccCypherMode,
                        gccCypherKeysizeRequired,
                        gccCypherVersion,
                        gccCypherBlocksize,
                        gccCypherGUID
                       );

  TfrmGridReport_Cypher = class(TfrmGridReport)
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
  private
    // Get the grid column index for a specific column
    function  ColumnToColIdx(column: TGridColumn_Cypher): integer;
    function  ColIdxToColumn(colIdx: integer): TGridColumn_Cypher;

    procedure SetCell(col: TGridColumn_Cypher; row: integer; value: string);

  public
    function CountDrivers(): integer; override;
    function CountImplementations(): integer; override;

    procedure SetupGrid(); override;
    procedure PopulateGrid(); override;
    function  IsColumnAdvanced(colIdx: integer): boolean; override;
  end;

implementation

{$R *.dfm}

uses
  SDUGeneral,
  SDUDialogs,
  OTFEFreeOTFE_U;

const
  TGridColumn_CypherTitle: array [TGridColumn_Cypher] of string = (
                                      'Driver title',
                                      'Driver version',
                                      'Driver GUID',
                                      'Driver device name',
                                      'Driver user mode name',
                                      'Driver kernel mode name',

                                      'Cypher title',
                                      'Cypher keysize',
                                      'Cypher mode',
                                      'Cypher keysize required',
                                      'Cypher version',
                                      'Cypher blocksize',
                                      'Cypher GUID'
                                     );

  ADVANCED_COLS_CYPHER: set of TGridColumn_Cypher = [
                                                    gccDriverGUID,
                                                    gccDriverDeviceName,
                                                    gccDriverUserModeName,
                                                    gccDriverKernelModeName,
                                                    gccCypherBlocksize,
                                                    gccCypherKeysizeRequired,
                                                    gccCypherGUID
                                                   ];



function TfrmGridReport_Cypher.ColumnToColIdx(column: TGridColumn_Cypher): integer;
begin
  Result:= ord(column);
end;

function TfrmGridReport_Cypher.ColIdxToColumn(colIdx: integer): TGridColumn_Cypher;
var
  i: TGridColumn_Cypher;
  retval: TGridColumn_Cypher;
begin
  retval := low(TGridColumn_Cypher);

  for i:=low(TGridColumn_Cypher) to high(TGridColumn_Cypher) do
    begin
    if (ColumnToColIdx(i) = colIdx) then
      begin
      retval := i;
      break;
      end;
    end;

  Result := retval;
end;

procedure TfrmGridReport_Cypher.SetCell(col: TGridColumn_Cypher; row: integer; Value: string);
var
  dispValue: string;
begin
  dispValue := Value;

  if (row <> GRID_HEADER_ROW) then
    begin
    if (col = gccCypherKeysizeUnderlying) then
      begin
      if (strtoint(Value) = 0) then
        begin
        dispValue := '(null)';
        end
      else if (strtoint(Value) = -1) then
        begin
        dispValue := '(any)';
        end;
      end;

    if (col = gccCypherKeysizeRequired) then
      begin
      if (strtoint(Value) = 0) then
        begin
        dispValue := '(null)';
        end
      else if (strtoint(Value) = -1) then
        begin
        dispValue := '(any)';
        end;
      end;

    if (col = gccCypherBlockSize) then
      begin
      if (strtoint(Value) = -1) then
        begin
        dispValue := '(any)';
        end;
      end;
    end;

  stgReport.Cells[ColumnToColIdx(col), row] := dispValue;
end;

procedure TfrmGridReport_Cypher.SetupGrid();
var
  i: TGridColumn_Cypher;
begin
  inherited;

  stgReport.ColCount  := ord(high(TGridColumn_Cypher)) - ord(low(TGridColumn_Cypher)) + 1;

  for i:=low(TGridColumn_Cypher) to high(TGridColumn_Cypher) do
    begin
    SetCell(i, GRID_HEADER_ROW, TGridColumn_CypherTitle[i]);
    end;

end;

procedure TfrmGridReport_Cypher.PopulateGrid();
var
  allData: TFreeOTFECypherDriverArray;
  i: integer;
  j: integer;
  currDriver: TFreeOTFECypherDriver;
  currImpl: TFreeOTFECypher_v3;
  currRow: integer;
begin
  inherited;

  if OTFEFreeOTFE.GetCypherDrivers(allData) then
    begin
    // +1 for the header row
    stgReport.RowCount := CountImplementations() + 1;

    currRow := 0;
    for i:=low(allData) to high(allData) do
      begin
      currDriver := allData[i];
      for j:=low(currDriver.Cyphers) to high(currDriver.Cyphers) do
        begin
        inc(currRow);
        currImpl := currDriver.Cyphers[j];

        SetCell(gccDriverTitle,             currRow, currDriver.Title);
        SetCell(gccDriverVersion,           currRow, OTFEFreeOTFE.VersionIDToStr(currDriver.VersionID));
        SetCell(gccDriverGUID,              currRow, GUIDToString(currDriver.DriverGUID));
        SetCell(gccDriverDeviceName,        currRow, currDriver.DeviceName);
        SetCell(gccDriverUserModeName,      currRow, currDriver.DeviceUserModeName);
        SetCell(gccDriverKernelModeName,    currRow, currDriver.DeviceKernelModeName);

        SetCell(gccCypherTitle,             currRow, currImpl.Title);
        SetCell(gccCypherKeysizeUnderlying, currRow, inttostr(currImpl.KeySizeUnderlying));
        SetCell(gccCypherKeysizeRequired,   currRow, inttostr(currImpl.KeySizeRequired));
        SetCell(gccCypherMode,              currRow, FreeOTFECypherModeTitle[currImpl.Mode]);
        SetCell(gccCypherVersion,           currRow, OTFEFreeOTFE.VersionIDToStr(currImpl.VersionID));
        SetCell(gccCypherBlocksize,         currRow, inttostr(currImpl.BlockSize));
        SetCell(gccCypherGUID,              currRow, GUIDToString(currImpl.CypherGUID));
        end;      
      end;
    end;

end;

function TfrmGridReport_Cypher.IsColumnAdvanced(colIdx: integer): boolean;
begin
  Result := (ColIdxToColumn(colIdx) in ADVANCED_COLS_CYPHER);
end;

function TfrmGridReport_Cypher.CountDrivers(): integer;
var
  retval: integer;
  allData: TFreeOTFECypherDriverArray;
begin
  inherited;

  retval := 0;
  if OTFEFreeOTFE.GetCypherDrivers(allData) then
    begin
    retval := high(allData) - low(allData) + 1;
    end;

  Result := retval;
end;

function TfrmGridReport_Cypher.CountImplementations(): integer;
var
  retval: integer;
  allData: TFreeOTFECypherDriverArray;
  i: integer;
  currDriver: TFreeOTFECypherDriver;
begin
  inherited;

  retval := 0;
  if OTFEFreeOTFE.GetCypherDrivers(allData) then
    begin
    for i:=low(allData) to high(allData) do
      begin
      currDriver := allData[i];
      retval := retval + currDriver.CypherCount;
      end;
    end;

  Result := retval;
end;

procedure TfrmGridReport_Cypher.FormCreate(Sender: TObject);
begin
  inherited;

  self.caption := 'Available Cyphers';
  lblTitle.Caption := 'The following cyphers are available for use:';

end;

procedure TfrmGridReport_Cypher.FormShow(Sender: TObject);
begin
  inherited;

  if (CountImplementations() <= 0) then
    begin
    SDUMessageDlg(
                  'No cypher algorithms could be found.'+SDUCRLF+
                  SDUCRLF+
                  'Please click "Drivers..." to install one or more cypher drivers',
                  mtError,
                  [mbOK],
                  0
                 );
    end;

end;

END.

